@@ -1,9 +1,9 @@
-from ..client import Client
-
 import pytest
 
+from ..client import Client
 
-class ClusterTest(object):
+
+class ClusterTest:
     Cluster = None
     kwargs = {}
 
