@@ -4,11 +4,20 @@ Most are taken from other test files and adapted.
 """
 import asyncio
 
-from distributed import Nanny, worker_client, Queue
-from distributed.client import wait
+import pytest
+
+from distributed import Client, Nanny, Queue, Scheduler, Worker, wait, worker_client
+from distributed.core import Status
 from distributed.metrics import time
-from distributed.nanny import Nanny
-from distributed.utils_test import gen_tls_cluster, inc, double, slowinc, slowadd
+from distributed.utils_test import (  # noqa: F401
+    cleanup,
+    double,
+    gen_tls_cluster,
+    inc,
+    slowadd,
+    slowinc,
+    tls_config,
+)
 
 
 @gen_tls_cluster(client=True)
@@ -90,16 +99,36 @@ async def test_nanny(c, s, a, b):
     assert result == 11
 
 
-@gen_tls_cluster(client=True)
-async def test_rebalance(c, s, a, b):
-    x, y = await c._scatter([1, 2], workers=[a.address])
-    assert len(a.data) == 2
-    assert len(b.data) == 0
+@gen_tls_cluster(
+    client=True,
+    Worker=Nanny,
+    worker_kwargs={"memory_limit": "1 GiB"},
+    config={"distributed.worker.memory.rebalance.sender-min": 0.3},
+)
+async def test_rebalance(c, s, *_):
+    # We used nannies to have separate processes for each worker
+    a, b = s.workers
+    assert a.startswith("tls://")
 
-    await c._rebalance()
+    # Generate 10 buffers worth 512 MiB total on worker a. This sends its memory
+    # utilisation slightly above 50% (after counting unmanaged) which is above the
+    # distributed.worker.memory.rebalance.sender-min threshold.
+    futures = c.map(lambda _: "x" * (2 ** 29 // 10), range(10), workers=[a])
+    await wait(futures)
 
-    assert len(a.data) == 1
-    assert len(b.data) == 1
+    # Wait for heartbeats
+    while s.memory.process < 2 ** 29:
+        await asyncio.sleep(0.1)
+
+    assert await c.run(lambda dask_worker: len(dask_worker.data)) == {a: 10, b: 0}
+
+    await c.rebalance()
+
+    ndata = await c.run(lambda dask_worker: len(dask_worker.data))
+    # Allow for some uncertainty as the unmanaged memory is not stable
+    assert sum(ndata.values()) == 10
+    assert 3 <= ndata[a] <= 7
+    assert 3 <= ndata[b] <= 7
 
 
 @gen_tls_cluster(client=True, nthreads=[("tls://127.0.0.1", 2)] * 2)
@@ -169,6 +198,41 @@ async def test_retire_workers(c, s, a, b):
     assert set(s.workers) == {b.worker_address}
 
     start = time()
-    while a.status != "closed":
+    while a.status != Status.closed:
         await asyncio.sleep(0.01)
         assert time() < start + 5
+
+
+@pytest.mark.asyncio
+async def test_security_dict_input_no_security(cleanup):
+    async with Scheduler(security={}) as s:
+        async with Worker(s.address, security={}) as w:
+            async with Client(s.address, security={}, asynchronous=True) as c:
+                result = await c.submit(inc, 1)
+                assert result == 2
+
+
+@pytest.mark.asyncio
+async def test_security_dict_input(cleanup):
+    conf = tls_config()
+    ca_file = conf["distributed"]["comm"]["tls"]["ca-file"]
+    client = conf["distributed"]["comm"]["tls"]["client"]["cert"]
+    worker = conf["distributed"]["comm"]["tls"]["worker"]["cert"]
+    scheduler = conf["distributed"]["comm"]["tls"]["scheduler"]["cert"]
+
+    async with Scheduler(
+        host="localhost",
+        security={"tls_ca_file": ca_file, "tls_scheduler_cert": scheduler},
+    ) as s:
+        assert s.address.startswith("tls://")
+        async with Worker(
+            s.address, security={"tls_ca_file": ca_file, "tls_worker_cert": worker}
+        ) as w:
+            assert w.address.startswith("tls://")
+            async with Client(
+                s.address,
+                security={"tls_ca_file": ca_file, "tls_client_cert": client},
+                asynchronous=True,
+            ) as c:
+                result = await c.submit(inc, 1)
+                assert result == 2
