@@ -1,13 +1,12 @@
 """ utilities for package version introspection """
 
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function
 
+import importlib
+import os
 import platform
 import struct
-import os
 import sys
-import importlib
-
 
 required_packages = [
     ("dask", lambda p: p.__version__),
@@ -33,7 +32,7 @@ scheduler_relevant_packages = set(pkg for pkg, _ in required_packages) | set(
 
 # notes to be displayed for mismatch packages
 notes_mismatch_package = {
-    "msgpack": "Variation is ok, as long as everything is above 0.6",
+    "msgpack": "Variation is ok, as long as everything is above 0.6"
 }
 
 
@@ -72,7 +71,7 @@ def get_system_info():
 
 
 def version_of_package(pkg):
-    """ Try a variety of common ways to get the version of a package """
+    """Try a variety of common ways to get the version of a package"""
     from contextlib import suppress
 
     with suppress(AttributeError):
@@ -85,7 +84,7 @@ def version_of_package(pkg):
 
 
 def get_package_info(pkgs):
-    """ get package versions for the passed required & optional packages """
+    """get package versions for the passed required & optional packages"""
 
     pversions = [("python", ".".join(map(str, sys.version_info)))]
     for pkg in pkgs:
@@ -168,26 +167,6 @@ def error_message(scheduler, workers, client, client_name="client"):
             if not isinstance(ws, set):
                 ws = {ws}
 
-            if name == "python":
-                majors = [tuple(version.split(".")[:2]) for version in {c, s} | ws]
-                if len(set(majors)) != 1:
-                    err_table = asciitable(
-                        ["Package", client_name, "scheduler", "workers"],
-                        [t for t in errs if t[0] == "python"],
-                    )
-                    out["error"] += f"Python major versions must match\n\n{err_table}\n"
-
-            if name == "lz4":
-                versions = [version for version in {c, s} | ws]
-                if any(versions) and not all(versions):
-                    err_table = asciitable(
-                        ["Package", client_name, "scheduler", "workers"],
-                        [t for t in errs if t[0] == "lz4"],
-                    )
-                    out[
-                        "error"
-                    ] += f"\nLZ4 must be installed everywhere or nowhere\n\n{err_table}\n"
-
     return out
 
 
